// GENERATED CONTENT - DO NOT EDIT
// Content was automatically extracted by Reffy into webref
// (https://github.com/w3c/webref)
// Source: Login Status API (https://w3c-fedid.github.io/login-status/)

enum LoginStatus {
  "logged-in",
  "logged-out",
};

[Exposed=Window, SecureContext]

interface NavigatorLogin {
  Promise<undefined> setStatus(LoginStatus status);
};

partial interface Navigator {
  [SecureContext] readonly attribute NavigatorLogin login;
};
