// GENERATED CONTENT - DO NOT EDIT
// Content was automatically extracted by Reffy into webref
// (https://github.com/w3c/webref)
// Source: SVG Paths (https://svgwg.org/specs/paths/)

[LegacyNoInterfaceObject, Exposed=Window]
interface SVGPathSegment {
  attribute DOMString type;
  attribute FrozenArray<float> values;
};

dictionary SVGPathDataSettings {
   boolean normalize = false;
};

interface mixin SVGPathData {
   sequence<SVGPathSegment> getPathData(optional SVGPathDataSettings settings = {});
   undefined setPathData(sequence<SVGPathSegment> pathData);
};

[Exposed=Window]
interface SVGPathElement : SVGGeometryElement {

  readonly attribute SVGAnimatedNumber pathLength;

  float getTotalLength();
  DOMPoint getPointAtLength(float distance);
  SVGPathSegment? getPathSegmentAtLength(float distance);
};

SVGPathElement includes SVGPathData;
